﻿#include "ChartsAndCamerasWnd.h"
#include "ui_ChartsAndCamerasWnd.h"
#include <QSettings>
#include <QCloseEvent>
#include "Group.h"
#include <QVBoxLayout>
#include <QTimer>
#include "chartline.h"
#include <QRandomGenerator>
#include <qDebug>

ChartsAndCamerasWnd::ChartsAndCamerasWnd(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::ChartsAndCamerasWnd),
    chartLine(new ChartLine(this)),
    chartLine1(new ChartLine(this)),
    chartLine2(new ChartLine(this))
{
    ui->setupUi(this);



    // 设置 ChartLine 的位置和大小
    chartLine->setParent(ui->Leftwidget);
    chartLine->setGeometry(0, 0, 560, 280);
    chartLine->setChartLabels("Combined Placement", "Number of Samples", "Device Align");

    // 设置 ChartLine1 的位置和大小
    chartLine1->setParent(ui->Leftwidget);
    chartLine1->setGeometry(0, 245, 560, 280);
    chartLine1->setChartLabels("Y placement", "Number of samples", "Y device align");

    // 设置 ChartLine2 的位置和大小
    chartLine2->setParent(ui->Leftwidget);
    chartLine2->setGeometry(0, 490, 560, 280);
    chartLine2->setChartLabels("Theta placement", "Number of samples", "Theta device align");

    // 初始化系列信息
    initializeSeriesInfo();

    // 连接所有复选框的 clicked 信号到通用槽函数
    QList<QCheckBox*> checkBoxes = {
        ui->GreenDiamondcheckBox,
        ui->YellowDiamondcheckBox,
        ui->BlackDiamondcheckBox,
        ui->BlueDiamondcheckBox,
        ui->BlueHorizontalLinecheckBox,
        ui->BlueVerticalBarcheckBox,
        ui->GreenVerticalBarcheckBox,
        ui->PurpleVerticalBarcheckBox,
        ui->RedDiamondcheckBox,
        ui->RedHorizontalLinecheckBox,
        ui->WhiteVerticalBarcheckBox
    };

    for (QCheckBox* cb : checkBoxes) {
        cb->setCheckable(true);

        connect(cb, &QCheckBox::clicked, this, &ChartsAndCamerasWnd::on_CheckBox_clicked);
    }



    initFrom();
}

ChartsAndCamerasWnd::~ChartsAndCamerasWnd()
{
    delete ui;
}




void ChartsAndCamerasWnd::initializeSeriesInfo()
{
    // 初始化系列信息，使用 DataSeries 结构体
    m_seriesMap["GreenDiamond"] = DataSeries{
        "GreenDiamond",
       QColor("#00FF00"), // 绿色
        QPixmap(":/images/Participation display/GreenDiamond.png"),
        QList<QPointF>() // 初始为空
    };

    m_seriesMap["YellowDiamond"] = DataSeries{
        "YellowDiamond",
        QColor("#FFFF00"), // 黄色
        QPixmap(":/images/Participation display/YellowDiamond.png"),
        QList<QPointF>()
    };

    m_seriesMap["BlackDiamond"] = DataSeries{
        "BlackDiamond",
        QColor("#000000"), // 黑色
        QPixmap(":/images/Participation display/BlackDiamond.png"),
        QList<QPointF>()
    };

    m_seriesMap["BlueDiamond"] = DataSeries{
        "BlueDiamond",
        QColor("#0000FF"), // 蓝色
        QPixmap(":/images/Participation display/BlueDiamond.png"),
        QList<QPointF>()
    };

    m_seriesMap["BlueHorizontalLine"] = DataSeries{
        "BlueHorizontalLine",
        QColor("#0000FF"),
        QPixmap(":/images/Participation display/Blue_horizontal_line.png"),
        QList<QPointF>()
    };

    m_seriesMap["BlueVerticalBar"] = DataSeries{
        "BlueVerticalBar",
        QColor("#0000FF"),
        QPixmap(":/images/Participation display/Blue_vertical_bar.png"),
        QList<QPointF>()
    };

    m_seriesMap["GreenVerticalBar"] = DataSeries{
        "GreenVerticalBar",
        QColor("#00FF00"),
        QPixmap(":/images/Participation display/Green_vertical_bar.png"),
        QList<QPointF>()
    };

    m_seriesMap["PurpleVerticalBar"] = DataSeries{
        "PurpleVerticalBar",
        QColor("#800080"),
        QPixmap(":/images/Participation display/Purple_vertical_bar.png"),
        QList<QPointF>()
    };

    m_seriesMap["RedDiamond"] = DataSeries{
        "RedDiamond",
        QColor("#FF0000"), // 红色
        QPixmap(":/images/Participation display/RedDiamond.png"),
        QList<QPointF>()
    };

    m_seriesMap["RedHorizontalLine"] = DataSeries{
        "RedHorizontalLine",
        QColor("#FF0000"),
        QPixmap(":/images/Participation display/Red_horizontal_line.png"),
        QList<QPointF>()
    };

    m_seriesMap["WhiteVerticalBar"] = DataSeries{
        "WhiteVerticalBar",
        QColor("#FFFFFF"), // 白色
        QPixmap(":/images/Participation display/White_vertical_bar.png"),
        QList<QPointF>()
    };
}




void ChartsAndCamerasWnd::on_CheckBox_clicked()
{
    // 获取发送信号的复选框
    QCheckBox* checkBox = qobject_cast<QCheckBox*>(sender());
    if (!checkBox)
        return;

    QString checkBoxName = checkBox->objectName();
    QString seriesName = checkBoxName;

    // 移除末尾的 "checkBox"
    if (seriesName.endsWith("checkBox")) {
        seriesName.chop(QString("checkBox").length());
    } else if (seriesName.endsWith("CheckBox")) {
        seriesName.chop(QString("CheckBox").length());
    }

    qDebug() << "Checkbox clicked:" << seriesName << (checkBox->isChecked() ? "Checked" : "Unchecked");

    if (!m_seriesMap.contains(seriesName)) {
        qDebug() << "No series info for:" << seriesName;
        return;
    }

    DataSeries &seriesInfo = m_seriesMap[seriesName];

    if (checkBox->isChecked()) {
        // 生成随机数据
        QList<QPointF> data;
        for (int i = 0; i <= 40; i += 4) {
            data.append(QPointF(i, QRandomGenerator::global()->bounded(-30, 80)));
        }
        seriesInfo.data = data;

        // 添加到 ChartLine 实例
        chartLine->addDataSeries(seriesInfo);
        chartLine1->addDataSeries(seriesInfo);
        chartLine2->addDataSeries(seriesInfo);
        qDebug() << "Added series:" << seriesName;
    }
    else {
        // 从 ChartLine 实例中移除
        chartLine->removeDataSeries(seriesName);
        chartLine1->removeDataSeries(seriesName);
        chartLine2->removeDataSeries(seriesName);
        qDebug() << "Removed series:" << seriesName;
    }
}





void ChartsAndCamerasWnd::initFrom() {
    QTimer *timer = new QTimer(this);
    connect(timer, &QTimer::timeout, this, &ChartsAndCamerasWnd::checkSettings);
    timer->start(100);

    ui->label_Percentage->setAlignment(Qt::AlignCenter);

    // 设置右下部分
    // loadSliderStates();

    initSliders();
    initLineEdits();

    connectSliderAndLineEdit(ui->verticalSlider, ui->lineEdit);
    connectSliderAndLineEdit(ui->verticalSlider_2, ui->lineEdit_2);
    connectSliderAndLineEdit(ui->verticalSlider_3, ui->lineEdit_3);
    connectSliderAndLineEdit(ui->verticalSlider_4, ui->lineEdit_4);
    connectSliderAndLineEdit(ui->verticalSlider_7, ui->lineEdit_7);
    connectSliderAndLineEdit(ui->verticalSlider_8, ui->lineEdit_8);

    QWidget *viewport = ui->scrollArea->viewport();
    QWidget *container = new QWidget(viewport);
    QHBoxLayout *layout = new QHBoxLayout(container);

    layout->setSpacing(16); // 设置Group之间的间隔距离
    layout->setContentsMargins(0, 0, 0, 0); // 设置布局的边距为0

    QStringList dispmodel;
    dispmodel << "P die align" << "D device align";
    Group *widget = new Group(1, ":/images/test_image/image_1.png", ":/images/test_image/image_2.png", dispmodel, this);
    layout->addWidget(widget);

    QStringList dispmodel1;
    dispmodel1 << "D device align" << "P die align";
    Group *widget_1 = new Group(2, ":/images/test_image/image_3.png", ":/images/test_image/image_4.png", dispmodel1, this);
    layout->addWidget(widget_1);

    QStringList dispmodel2;
    dispmodel2 << "B device align" << "P die align";
    Group *widget_2 = new Group(3, ":/images/test_image/image_5.png", ":/images/test_image/image_6.png", dispmodel2, this);
    layout->addWidget(widget_2);

    QStringList dispmodel3;
    dispmodel3 << "U die align" << "D device align";
    Group *widget_3 = new Group(4, ":/images/test_image/image_7.png", ":/images/test_image/image_8.png", dispmodel3, this);
    layout->addWidget(widget_3);

    Group *widget_4 = new Group(5, ":/images/test_image/image.png", ":/images/test_image/image_2.png", dispmodel, this);
    layout->addWidget(widget_4);

    // 设置控件的最小高度和最大宽度
    int minHeight = 162;
    int maxWidth = 244;
    QWidgetList widgets = {widget, widget_1, widget_2, widget_3, widget_4};
    for (QWidget *w : widgets) {
        w->setMinimumHeight(minHeight);
        w->setMaximumWidth(maxWidth);
    }

    container->setLayout(layout);
    ui->scrollArea->setWidget(container);

    // 确保滚动条策略正确设置
    ui->scrollArea->setHorizontalScrollBarPolicy(Qt::ScrollBarAsNeeded);
    ui->scrollArea->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);

    // 调整滚动区域的大小以适应内容
    ui->scrollArea->resize(529, 172);

    ui->Operatewidget->setMouseTracking(true);
}

void ChartsAndCamerasWnd::initSliders() {
    QList<QSlider*> sliders = {
        ui->verticalSlider, ui->verticalSlider_2, ui->verticalSlider_3,
        ui->verticalSlider_4, ui->verticalSlider_7, ui->verticalSlider_8
    };
    QList<int> initialValues = {11, 22, 9, 11, 0, 0};

    for (int i = 0; i < sliders.size(); ++i) {
        sliders[i]->setMinimum(0);
        sliders[i]->setMaximum(100);
        sliders[i]->setValue(initialValues[i]);
    }
}

void ChartsAndCamerasWnd::initLineEdits() {
    QList<QLineEdit*> lineEdits = {
        ui->lineEdit, ui->lineEdit_2, ui->lineEdit_3,
        ui->lineEdit_4, ui->lineEdit_7, ui->lineEdit_8
    };

    for (QLineEdit* lineEdit : lineEdits) {
        lineEdit->setAlignment(Qt::AlignCenter);
    }
}

void ChartsAndCamerasWnd::connectSliderAndLineEdit(QSlider* slider, QLineEdit* lineEdit)
{
    QIntValidator* validator = new QIntValidator(slider->minimum(), slider->maximum(), lineEdit);
    lineEdit->setValidator(validator);

    connect(slider, &QSlider::valueChanged, [lineEdit](int value) {
        lineEdit->setText(QString::number(value));
    });

    connect(lineEdit, &QLineEdit::textChanged, [slider](const QString &text) {
        bool ok;
        int value = text.toInt(&ok);
        if (ok && value >= slider->minimum() && value <= slider->maximum()) {
            slider->setValue(value);
        } else if (!text.isEmpty()) {
            int closestValue = qMin(qMax(text.toInt(&ok), slider->minimum()), slider->maximum());
            slider->setValue(closestValue);
        }
    });
}

void ChartsAndCamerasWnd::checkSettings() {


    QSettings settings("YourCompany", "YourApplication_");
    int groupId = settings.value("GroupId", 0).toInt();
    int index = settings.value("Index", 0).toInt();

    if (groupId != lastGroupId || index != lastIndex) {
        lastGroupId = groupId;
        lastIndex = index;
        loadSettings();
    }
}

void ChartsAndCamerasWnd::loadSettings()
{
    QSettings settings("YourCompany", "YourApplication_");
    int groupId = settings.value("GroupId", 0).toInt();
    int index = settings.value("Index", 0).toInt();
    loadGroupSettings(groupId, index);
}

void ChartsAndCamerasWnd::loadGroupSettings(int Id, int Index) {
    QSettings settings("YourOrganization", "YourApplication");
    settings.beginGroup(QString::number(Id));
    QString imagePath1 = settings.value("ImagePath1").toString();
    QString imagePath2 = settings.value("ImagePath2").toString();
    QStringList textList = settings.value("TextList").toStringList();
    settings.endGroup();

    QSize size = ui->Operatewidget->size();

    QPixmap newPixmap;
    if (Index == 1) {
        newPixmap = QPixmap(imagePath1);
    } else if (Index == 2) {
        newPixmap = QPixmap(imagePath2);
    } else {
        newPixmap = QPixmap(":/images/test_image/image.png");
    }

    QPixmap scaledPixmap = newPixmap.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    ui->Operatewidget->setPixmap(scaledPixmap);

    // 更新当前图片的成员变量
    currentPixmap = scaledPixmap;
    scaleFactor = 1.0;
    ui->label_Percentage->setText("100%");

    ui->DatacomboBox->clear();
    ui->DatacomboBox->addItems(textList);
    // connect(ui->DatacomboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(Demo001_3::onComboBoxIndexChanged(int)));
}

void ChartsAndCamerasWnd::on_ZoomUpButton_clicked()
{
    if (scaleFactor < 3.0) {
        scaleFactor *= 1.1;
    }

    int newWidth = currentPixmap.width() * scaleFactor;
    int newHeight = currentPixmap.height() * scaleFactor;

    QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);

    ui->Operatewidget->setPixmap(scaledImage);

    double percentage = scaleFactor * 100;
    QString percentageStr = QString::number((int)percentage);
    ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
}

void ChartsAndCamerasWnd::on_ZoomOutButton_clicked()
{
    if (scaleFactor > 0.25) {
        scaleFactor *= 0.9;
    }

    int newWidth = currentPixmap.width() * scaleFactor;
    int newHeight = currentPixmap.height() * scaleFactor;

    QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);

    ui->Operatewidget->setPixmap(scaledImage);

    double percentage = scaleFactor * 100;
    QString percentageStr = QString::number((int)percentage);
    ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
}

void ChartsAndCamerasWnd::wheelEvent(QWheelEvent *event)
{
    if (ui->Operatewidget->rect().contains(ui->Operatewidget->mapFromGlobal(event->globalPos()))) {
        if (event->angleDelta().y() > 0) {
            if (scaleFactor < 3.0) {
                scaleFactor *= 1.1;
            }
        } else {
            if (scaleFactor > 0.25) {
                scaleFactor *= 0.9;
            }
        }

        int newWidth = currentPixmap.width() * scaleFactor;
        int newHeight = currentPixmap.height() * scaleFactor;

        QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        ui->Operatewidget->setPixmap(scaledImage);

        double percentage = scaleFactor * 100;
        QString percentageStr = QString::number((int)percentage);
        ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
    }

    QMainWindow::wheelEvent(event);
}

void ChartsAndCamerasWnd::showEvent(QShowEvent *event) {
    QMainWindow::showEvent(event);
    loadSettings();
    // initFrom();
}

void ChartsAndCamerasWnd::hideEvent(QHideEvent *event) {
    QMainWindow::hideEvent(event);
}



