#include "SingleCameraOperationWnd.h"
#include "ui_SingleCameraOperationWnd.h"
#include <QSettings>
#include <QCloseEvent>
#include "Group.h"
#include <QVBoxLayout>
#include <QDebug>
#include <QTimer>

SingleCameraOperationWnd::SingleCameraOperationWnd(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::SingleCameraOperationWnd), scaleFactor(1.0)
{
    ui->setupUi(this);

    initFrom();
    // connect(this, &SingleCameraOperationWnd::closeEvent, this, &SingleCameraOperationWnd::saveSliderStates);
}

SingleCameraOperationWnd::~SingleCameraOperationWnd()
{
    delete ui;
}

void SingleCameraOperationWnd::initFrom() {
    QTimer *timer = new QTimer(this);
    connect(timer, &QTimer::timeout, this, &SingleCameraOperationWnd::checkSettings);
    timer->start(100);

    ui->label_Percentage->setAlignment(Qt::AlignCenter);

    // 设置右下部分
    loadSliderStates();

    initSliders();
    initLineEdits();

    connectSliderAndLineEdit(ui->verticalSlider, ui->lineEdit);
    connectSliderAndLineEdit(ui->verticalSlider_2, ui->lineEdit_2);
    connectSliderAndLineEdit(ui->verticalSlider_3, ui->lineEdit_3);
    connectSliderAndLineEdit(ui->verticalSlider_4, ui->lineEdit_4);
    connectSliderAndLineEdit(ui->verticalSlider_5, ui->lineEdit_5);
    connectSliderAndLineEdit(ui->verticalSlider_6, ui->lineEdit_6);

    // 设置右上部分
    QWidget *viewport = ui->scrollArea->viewport();
    QWidget *container = new QWidget(viewport);
    QVBoxLayout *layout = new QVBoxLayout(container);

    layout->setSpacing(16); // 设置Group之间的间隔距离
    layout->setMargin(0);

    QStringList dispmodel;
    dispmodel<<("P die align")<<("D device align");
    Group *widget = new Group(1, ":/images/test_image/image_1.png", ":/images/test_image/image_2.png", dispmodel, this);
    layout->addWidget(widget);

    QStringList dispmodel1;
    dispmodel1<<("D device align")<<("P die align");
    Group *widget_1 = new Group(2, ":/images/test_image/image_3.png", ":/images/test_image/image_4.png", dispmodel1, this);
    layout->addWidget(widget_1);

    QStringList dispmodel2;
    dispmodel2<<("B device align")<<("P die align");
    Group *widget_2 = new Group(3, ":/images/test_image/image_5.png", ":/images/test_image/image_6.png", dispmodel2, this);
    layout->addWidget(widget_2);

    QStringList dispmodel3;
    dispmodel3<<("U die align")<<("D device align");
    Group *widget_3 = new Group(4, ":/images/test_image/image_7.png", ":/images/test_image/image_8.png", dispmodel3, this);
    layout->addWidget(widget_3);

    Group *widget_4 = new Group(5, ":/images/test_image/image.png", ":/images/test_image/image_2.png", dispmodel, this);
    layout->addWidget(widget_4);

    int minHeight = 162;
    int maxWidth = 244;
    QWidgetList widgets = {widget, widget_1, widget_2, widget_3, widget_4};
    for (QWidget *w : widgets) {
        w->setMinimumHeight(minHeight);
        w->setMaximumWidth(maxWidth);
    }

    container->setLayout(layout);
    ui->scrollArea->setWidget(container);
    ui->scrollArea->resize(261, 700);

    ui->Operatewidget->setMouseTracking(true);
}

void SingleCameraOperationWnd::closeEvent(QCloseEvent *event) {
    // 保存滑块状态
    saveSliderStates();

    // 确保调用基类的 closeEvent 处理其他关闭逻辑
    QMainWindow::closeEvent(event);
}

void SingleCameraOperationWnd::initSliders() {
    QList<QSlider*> sliders = {
        ui->verticalSlider, ui->verticalSlider_2, ui->verticalSlider_3,
        ui->verticalSlider_4, ui->verticalSlider_5, ui->verticalSlider_6
    };
    QList<int> initialValues = {11, 22, 9, 11, 0, 0};

    for (int i = 0; i < sliders.size(); ++i) {
        sliders[i]->setMinimum(0);
        sliders[i]->setMaximum(100);
        sliders[i]->setValue(initialValues[i]);
    }
}

void SingleCameraOperationWnd::initLineEdits() {
    QList<QLineEdit*> lineEdits = {
        ui->lineEdit, ui->lineEdit_2, ui->lineEdit_3,
        ui->lineEdit_4, ui->lineEdit_5, ui->lineEdit_6
    };

    for (QLineEdit* lineEdit : lineEdits) {
        lineEdit->setAlignment(Qt::AlignCenter);
    }
}

void SingleCameraOperationWnd::saveSliderStates() {
    QSettings settings("YourCompany", "YourApp");
    settings.beginGroup("SliderStates");
    settings.setValue("verticalSlider", ui->verticalSlider->value());
    settings.setValue("verticalSlider_2", ui->verticalSlider_2->value());
    settings.setValue("verticalSlider_3", ui->verticalSlider_3->value());
    settings.setValue("verticalSlider_4", ui->verticalSlider_4->value());
    settings.setValue("verticalSlider_5", ui->verticalSlider_5->value());
    settings.setValue("verticalSlider_6", ui->verticalSlider_6->value());
    settings.endGroup();
}

void SingleCameraOperationWnd::loadSliderStates() {
    QSettings settings("YourCompany", "YourApp");
    settings.beginGroup("SliderStates");
    ui->verticalSlider->setValue(settings.value("verticalSlider", 11).toInt());
    ui->verticalSlider_2->setValue(settings.value("verticalSlider_2", 22).toInt());
    ui->verticalSlider_3->setValue(settings.value("verticalSlider_3", 9).toInt());
    ui->verticalSlider_4->setValue(settings.value("verticalSlider_4", 11).toInt());
    ui->verticalSlider_5->setValue(settings.value("verticalSlider_5", 0).toInt());
    ui->verticalSlider_6->setValue(settings.value("verticalSlider_6", 0).toInt());
    settings.endGroup();
}

// QSlider 和 QLineEdit 的联动
void SingleCameraOperationWnd::connectSliderAndLineEdit(QSlider* slider, QLineEdit* lineEdit)
{
    QIntValidator* validator = new QIntValidator(slider->minimum(), slider->maximum(), lineEdit);
    lineEdit->setValidator(validator);

    connect(slider, &QSlider::valueChanged, [lineEdit](int value) {
        lineEdit->setText(QString::number(value));
    });

    connect(lineEdit, &QLineEdit::textChanged, [slider](const QString &text) {
        bool ok;
        int value = text.toInt(&ok);
        if (ok && value >= slider->minimum() && value <= slider->maximum()) {
            slider->setValue(value);
        } else if (!text.isEmpty()) {
            int closestValue = qMin(qMax(text.toInt(&ok), slider->minimum()), slider->maximum());
            slider->setValue(closestValue);
        }
    });
}

void SingleCameraOperationWnd::checkSettings() {
    QSettings settings("YourCompany", "YourApplication_");
    int groupId = settings.value("GroupId", 0).toInt();
    int index = settings.value("Index", 0).toInt();

    if (groupId != lastGroupId || index != lastIndex) {
        lastGroupId = groupId;
        lastIndex = index;
        loadSettings();
    }
}

void SingleCameraOperationWnd::loadSettings()
{
    QSettings settings("YourCompany", "YourApplication_");
    int groupId = settings.value("GroupId", 0).toInt();
    int index = settings.value("Index", 0).toInt();
    loadGroupSettings(groupId, index);
}

void SingleCameraOperationWnd::loadGroupSettings(int Id, int Index) {
    QSettings settings("YourOrganization", "YourApplication");
    settings.beginGroup(QString::number(Id));
    QString imagePath1 = settings.value("ImagePath1").toString();
    QString imagePath2 = settings.value("ImagePath2").toString();
    QStringList textList = settings.value("TextList").toStringList();
    settings.endGroup();

    QSize size = ui->Operatewidget->size();

    QPixmap newPixmap;
    if (Index == 1) {
        newPixmap = QPixmap(imagePath1);
    } else if (Index == 2) {
        newPixmap = QPixmap(imagePath2);
    } else {
        newPixmap = QPixmap(":/images/test_image/image.png");
    }

    QPixmap scaledPixmap = newPixmap.scaled(size, Qt::KeepAspectRatio, Qt::SmoothTransformation);
    ui->Operatewidget->setPixmap(scaledPixmap);

    // 更新当前图片的成员变量
    currentPixmap = scaledPixmap;
    scaleFactor = 1.0;
    ui->label_Percentage->setText("100%");

    ui->DatacomboBox->clear();
    ui->DatacomboBox->addItems(textList);
    // qDebug() << "ComboBox items added:" << ui->DatacomboBox->count();
    // connect(ui->DatacomboBox, SIGNAL(currentIndexChanged(int)), this, SLOT(SingleCameraOperationWnd::onComboBoxIndexChanged(int)));
}

void SingleCameraOperationWnd::on_ZoomUpButton_clicked() {
    if (scaleFactor < 3.0) {
        scaleFactor *= 1.1;
    }

    int newWidth = currentPixmap.width() * scaleFactor;
    int newHeight = currentPixmap.height() * scaleFactor;

    QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);

    ui->Operatewidget->setPixmap(scaledImage); // 这里传递缩放后的图片

    double percentage = (scaleFactor * 100);
    QString percentageStr = QString::number((int)percentage);
    ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
}

void SingleCameraOperationWnd::on_ZoomOutButton_clicked() {
    if (scaleFactor > 0.25) {
        scaleFactor *= 0.9;
    }

    int newWidth = currentPixmap.width() * scaleFactor;
    int newHeight = currentPixmap.height() * scaleFactor;

    QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);

    ui->Operatewidget->setPixmap(scaledImage); // 这里传递缩放后的图片

    double percentage = scaleFactor * 100;
    QString percentageStr = QString::number((int)percentage);
    ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
}

void SingleCameraOperationWnd::wheelEvent(QWheelEvent *event) {
    // 检查鼠标事件是否发生在 Operatewidget 上
    if (ui->Operatewidget->rect().contains(ui->Operatewidget->mapFromGlobal(event->globalPos()))) {
        if (event->angleDelta().y() > 0) {
            if (scaleFactor < 3.0) {
                scaleFactor *= 1.1;
            }
        } else {
            if (scaleFactor > 0.25) {
                scaleFactor *= 0.9;
            }
        }

        int newWidth = currentPixmap.width() * scaleFactor;
        int newHeight = currentPixmap.height() * scaleFactor;

        QPixmap scaledImage = currentPixmap.scaled(newWidth, newHeight, Qt::KeepAspectRatio, Qt::SmoothTransformation);
        ui->Operatewidget->setPixmap(scaledImage); // 这里传递缩放后的图片

        double percentage = scaleFactor * 100;
        QString percentageStr = QString::number((int)percentage);
        ui->label_Percentage->setText(QString("%1%").arg(percentageStr));
    }

    QMainWindow::wheelEvent(event);
}

void SingleCameraOperationWnd::onComboBoxIndexChanged(int index) {
    // 根据index获取图片路径
    QString imagePath = getImagePathFromIndex(index);
    if (!imagePath.isEmpty()) {
        QPixmap newPixmap(imagePath);
        QPixmap scaledPixmap = newPixmap.scaled(ui->Operatewidget->size(), Qt::KeepAspectRatio, Qt::SmoothTransformation);
        ui->Operatewidget->setPixmap(scaledPixmap);
        // 更新当前图片的成员变量
        currentPixmap = scaledPixmap;
        scaleFactor = 1.0;
        ui->label_Percentage->setText("100%");
    }
}

QString SingleCameraOperationWnd::getImagePathFromIndex(int index) {
    QStringList imagePaths = {
        ":/images/test_image/image.png",
        ":/images/test_image/image_2.png",
    };
    if (index >= 0 && index < imagePaths.size()) {
        return imagePaths[index];
    }
    return ":/images/test_image/image.png";
}

void SingleCameraOperationWnd::showEvent(QShowEvent *event) {
    QMainWindow::showEvent(event);
    loadSettings();
}

void SingleCameraOperationWnd::hideEvent(QHideEvent *event) {
    QMainWindow::hideEvent(event);
}

